/* Copyright (c) 2012 by Al Williams

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

This software is intended to communicate with a GP9 device. Note that the above license covers,
However, this software does not include the firmware that resides inside the GP9. The above license
covers THIS software and not the GP9 firmware, which is not included in this distribution.

More info: http://www.awce.com/gp9.htm
al.williams@awce.com

*/

#include "gp9.h"
#include "gp3linux.h"


bool gp9::isOpen()
{
    return portopen;
}

bool gp9::open()
{
    portopen=gp3openport(port.toAscii())!=-1;
    if (!portopen) error=true; else error=false;
    return portopen;
}

bool gp9::close()
{
    gp3closeport();
    error=portopen=false;
    return true;
}

void gp9::setpwm(int chan, int pwm)
{
    write(chan & 7, pwm);
}

void gp9::setsource(int mask)
{
    if (!portopen)
        if (!open()) return;
    write(8, mask);
}

void gp9::setjam(int mask)
{
    write(0xA,mask);
}

void gp9::setoe(int mask)
{
    write(0xD,mask);
}

int gp9::readpwm(int chan)
{
    write(0x20+chan);
    return read();
}

int gp9::readad(int chan)
{
    write(0x30+chan);
    return read();
}

void gp9::write(int byte)
{
    if (!portopen)
        if (!open()) return;
    gp3write(byte);
}
void gp9::write(int b1, int b2)
{
    if (b2&1) b1|=0x40;
    b2>>=1;
    b2|=0x80;
    write(b1 &0xFF);
    write(b2 &0xFF);
}

int gp9::read(void)
{
    int c=gp3read();
    if (c==0x100) error=true;
    return c==0x100?-1:c;
}


int gp9::getoe(void)
{
    write(0x2d);
    return read();
}

int gp9::getsource(void)
{
    write(0x28);
    return read();
}




