/* Copyright (c) 2012 by Al Williams

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

This software is intended to communicate with a GP9 device. Note that the above license covers,
However, this software does not include the firmware that resides inside the GP9. The above license
covers THIS software and not the GP9 firmware, which is not included in this distribution.

More info: http://www.awce.com/gp9.htm
al.williams@awce.com

*/


#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "gp9.h"
#include <QMessageBox>
#include <QDir>
#include <QSettings>


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    int i;
    board=NULL;
    ui->setupUi(this);
    // populate COM ports
#if !defined(Q_OS_WIN)
    // note System needed for Dev files!
    QDir dev("/dev","tty[SU]*",QDir::Name,QDir::Files|QDir::Readable|QDir::Writable|QDir::System);
    QStringList devices;
    devices=dev.entryList();
    ui->comPort->addItems(devices);
#else
    // just stick a few in, the user can edit if needed
    ui->comPort->addItem("COM1");
    ui->comPort->addItem("COM2");
    ui->comPort->addItem("COM3");
    ui->comPort->addItem("COM4");
#endif
// get saved port
    QSettings set("awce.com","gp9demo");
#if defined(Q_OS_WIN)
    QString portsel=set.value("option/port","COM1").toString();
#else
    QString portsel=set.value("option/port","ttyS0").toString();
#endif
    // if we have one make sure it is in the list and select it
    if (!portsel.isEmpty())
    {
        int idx;
        idx=ui->comPort->findText(portsel);
        if (idx<0)
        {
            ui->comPort->addItem(portsel);
            idx=ui->comPort->findText(portsel);
        }
        if (idx>=0) ui->comPort->setCurrentIndex(idx);
    }
    // storing the widgets in an array is more flexible than scanning through the widget heirarchy
    // since that changes every time you redraw the layout in designer
    oes<<ui->oe0<<ui->oe0_2<<ui->oe0_3<<ui->oe0_4<<ui->oe0_5<<ui->oe0_6<<ui->oe0_7<<ui->oe0_8;
    selects<<ui->pot0<<ui->pot0_2<<ui->pot0_3<<ui->pot0_4<<ui->pot0_5<<ui->pot0_6<<ui->pot0_7<<ui->pot0_8;
    sliders<<ui->verticalSlider<<ui->verticalSlider_2<<ui->verticalSlider_3<<ui->verticalSlider_4<<ui->verticalSlider_5<<ui->verticalSlider_6<<ui->verticalSlider_7<<ui->verticalSlider_8;
    lcds<<ui->v0<<ui->v1<<ui->v2<<ui->v3<<ui->v4;
    uiEnable(false);
    // set up maps so that all the OE and select and sliders generate one event
    for (i=0;i<oes.count();i++)
    {
        oemap.setMapping(oes[i],i);
        connect(oes[i],SIGNAL(clicked()),&oemap,SLOT(map()));
        selmap.setMapping(selects[i],i);
        connect(selects[i],SIGNAL(clicked()),&selmap,SLOT(map()));
        slidemap.setMapping(sliders[i],i);
        connect(sliders[i],SIGNAL(valueChanged(int)),&slidemap,SLOT(map()));
    }
    // connect the one mapped event to each slot
    connect(&oemap,SIGNAL(mapped(int)),this,SLOT(oe_clicked(int)));
    connect(&selmap,SIGNAL(mapped(int)),this,SLOT(sel_clicked(int)));
    connect(&slidemap,SIGNAL(mapped(int)),this,SLOT(slide_change(int)));



}

MainWindow::~MainWindow()
{
    delete ui;
    if (board) board->close();

}

// enable or disable the UI (can't change things while disconnected)
void MainWindow::uiEnable(bool flag)
   {
   QCheckBox *box;
   QSlider *slide;
   foreach(box,oes)
      {
          box->setEnabled(flag);
      }
   for (int i=0;i<5;i++)  // last 3 are always disabled
   {
       selects[i]->setEnabled(flag);
   }
   foreach(slide, sliders)
   {
       slide->setEnabled(flag);
   }
}

// Every timer tick come here and update the voltmeters
void MainWindow::timerEvent(QTimerEvent *event)
{
    Q_UNUSED(event);
    if (board && board->isOpen())
    {
        for (int i=0;i<5;i++)  // just do each one
        {
            float v;
            v=5.0/256;   // EU conversion factor
            v*=board->readad(i);
            lcds[i]->display(v);
        }
    }
}


// This takes a mask in bits and maps it to the OE or Sel checkboxes
// invert is true if a 1 bit is an uncheck
void MainWindow::bitset(int bits, QList<QCheckBox *> box, bool invert)
{
    int mask=1;
    int i=0;
    while (mask<=0x80)
       {
           bool b=(mask & bits)?true:false;
           if (invert) b=!b;
           box[i++]->setChecked(b);
           mask<<=1;
       }
}

// Open the port and get started
// Or stop (we key off the board's open state)
void MainWindow::on_openBtn_clicked()
{
    if (board==NULL || !board->isOpen())
    {
        board=new gp9(ui->comPort->currentText());
        QSettings set("awce.com","gp9demo");
        set.setValue("option/port",ui->comPort->currentText());
        if (!board->open())
        {
            QMessageBox::critical(this,tr("Error"),tr("Can't open port"));
            return;
        }
        // change button state
        ui->openBtn->setText("&Close");
        timerid=startTimer(200);
        uiEnable();
        ui->comPort->setEnabled(false);

        // Set up checkboxes to initial state
        bitset(board->getsource(),selects);
        bitset(board->getoe(),oes,true);

        // set sliders to initial values
        for (int i=0;i<sliders.count();i++)
        {
            int val;
            val=board->readpwm(i);
            sliders[i]->setValue(val);
            sliders[i]->setStatusTip(QString::number(val));

        }
    }
      else  // board was open so close it
        {
            killTimer(timerid);  // stop the timer
            ui->openBtn->setText("&Open");  // put the button back to open
            board->close();      // close the board and delete it
            delete board;
            uiEnable(false);   // turn off most of the UI
            ui->comPort->setEnabled(true);  // but let us pick the comport for next time
        }

    return;
}


// slider select clicked

void MainWindow::sel_clicked(int i)
{

    if (selects[i]->checkState()) selmask|=(1<<i); else selmask&=~(1<<i);
    board->setsource(selmask);
}

// output enable clicked
void MainWindow::oe_clicked(int i)
{

    if (oes[i]->checkState()) oemask&=~(1<<i); else oemask|=(1<<i);
    board->setoe(oemask);
}

// slider changed
void MainWindow::slide_change(int i)
{
    int value=sliders[i]->value();
    sliders[i]->setStatusTip(QString::number(value));
    board->setpwm(i,value);
}
